#include <math.h>

#include <QtCore/QFile>
#include <QtGui/QMouseEvent>
#include <QtGui/QMessageBox>
#include <QtGui/QImage>

#include "Vector3D.h"
#include "glwidget.h"

GLWidget::GLWidget(QWidget *parent, QGLFormat format)
: QGLWidget(format, parent)
, m_mesh(0)
, m_texMode(SPHERE)
, m_textureId(0)
, m_xTrans(0)
, m_yTrans(0)
, m_zTrans(0)
, m_fovy(60.0f)
, m_zNear(0.1f)
, m_zFar(10.0f)
{
}       

GLWidget::~GLWidget()
{
	delete m_mesh;
}

bool GLWidget::loadTexture(const QString &filename) 
{
	if( 0 != m_mesh )
	{
		m_mesh->assignTexture(filename.toAscii().data());
		updateGL();
		return true;
	}
	return false;
}

bool GLWidget::loadModel(const QString &filename) 
{
	delete m_mesh;
	m_mesh = new Mesh( filename.toAscii().data() );
	if( !m_mesh->success() )
	{
		cerr << "Failed reading model\n";
		return false;
	}
	else
	{
		m_mesh->loadTextureImages();
		setTexMode( static_cast<int>(m_texMode) );
		return true;
	}
}

void GLWidget::setTexMode(int mode) 
{
	m_texMode = static_cast<TexMode>(mode);
	if( 0 != m_mesh )
	{
		switch(m_texMode)
		{
		case SPHERE:
			m_mesh->generateSphereTexCoords();
			break;
		case CYLINDER:
			m_mesh->generateCylinderTexCoords();
			break;
		}
	}
	updateGL();
}

void GLWidget::initializeGL() 
{ 
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glDisable(GL_BLEND);
	glEnable(GL_NORMALIZE);
	glDisable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);     
	glShadeModel(GL_SMOOTH);
	glEnable(GL_TEXTURE_2D);

	if( 0 != m_mesh )
	{
		m_mesh->loadTextureImages();
	}

	glLoadIdentity();     
	m_xTrans = m_yTrans = m_zTrans = 0.0f;
	glGetFloatv(GL_MODELVIEW_MATRIX, m_rotationMatrix);


	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE);
}


void GLWidget::resizeGL(int w, int h) 
{   
	glViewport( 0, 0, static_cast<GLsizei>(w), static_cast<GLsizei>(h) );  
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();  

	float r =  w / static_cast<float>(h);

	gluPerspective(60, r, 0.1, 10);

	glMatrixMode(GL_MODELVIEW);  

}

void GLWidget::paintGL() 
{   
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);    

	glPushMatrix();
	glLoadIdentity();
	gluLookAt(0, 0, 3, 0, 0, 0, 0, 1, 0);        

	glTranslatef(m_xTrans, m_yTrans, m_zTrans);
	glMultMatrixf(m_rotationMatrix);

	if( 0 != m_mesh )
	{
		float scale = (m_mesh->getBBoxMax() - m_mesh->getBBoxMin()).length() / 3;
		glScalef(1/scale, 1/scale, 1/scale);
		Vector3D center = (m_mesh->getBBoxMax() + m_mesh->getBBoxMin()) / 2;
		glTranslatef(-center[0], -center[1], -center[2]);
		m_mesh->draw();
	}

	glPopMatrix();
}

void GLWidget::mouseMoveEvent(QMouseEvent *e) 
{
	float normf = tan(m_fovy/360*M_PI)*(m_zFar+m_zNear)/(width()<height()?width():height());
	float dx = normf * static_cast<float>(e->x() - m_lastPos.x()); 
	float dy = normf * static_cast<float>(e->y() - m_lastPos.y());

	bool ctrl_key = e->modifiers() & Qt::MetaModifier;		// only needed for Mac OS X, but doesn't hurt on other OSes

	if ( (e->buttons() & Qt::RightButton) || ctrl_key )
	{
		m_zTrans += dy;
	} 
	else if (e->buttons() & Qt::MidButton)
	{
		m_xTrans += dx;
		m_yTrans -= dy;
	} 
	else if (e->buttons() & Qt::LeftButton)
	{
		glPushMatrix();
		glLoadIdentity();
		glRotatef(40*dx,0.0, 1.0, 0.0);
		glRotatef(40*dy,1.0, 0.0, 0.0);
		glMultMatrixf(m_rotationMatrix);
		glGetFloatv(GL_MODELVIEW_MATRIX,m_rotationMatrix);
		glPopMatrix();
	}
	m_lastPos = e->pos();
	e->accept();
	updateGL();
}

void GLWidget::mousePressEvent(QMouseEvent *e) 
{
	m_lastPos = e->pos();
}
